unit SoniqueVisTemplateUnit;

Interface
//====================================================================================
//
//	Sonique Visual Plugin API
//
//        (c) 1999 Sonique Team, edited by Rajesh Singh (shock@ihug.com.au)
//
//        Delphi Translation By George Boudouris 2009, (georgebou@hotmail.com)
//====================================================================================*/

uses
{  SysUtils,
  Classes,}
  Sonique_Vis_Delphi_Header;

//====================================================================================
//	Definitions of the functions for Plug-In (don't change)
//====================================================================================*/

Procedure Initialize; cdecl;
Function Render(Video:PCardinal;width,height,pitch:Integer;pVD:PVisData):Boolean; cdecl;
Function SaveSettings(FileName:PChar):Boolean; cdecl;
Function OpenSettings(FileName:PChar):Boolean; cdecl;
Function DeInit:Boolean; cdecl;
Function Clicked (x,y,buttons:Integer):Boolean; cdecl;
Function ReceiveQueryInterface(PluginInterface:QueryInterface):Boolean; cdecl;

//====================================================================================
//	The actual details for your plugin
//
//	1. Reserved (leave as NULL)
//	2. Plug-In Name (char*, the name you want displayed when your plug-in is playing)
//	3. Options (VI_WAVEFORM - if you need the waveform PCM data
//	            VI_SPECTRUM - if you need the spectrum FFT data
//	   	    SONIQUEVISPROC - if you want user defined blurs to effect your plugin)
//	4. Name of the Initialize function (leave as is)
//	5. Name of the Render function (leave as is)
//	6. Name of the SaveSettings function (leave as is)
//	7. Name of the OpenSettings function (leave as is)
//====================================================================================*/

Const
     plugin:TVisInfo=(Version               : 1;
                      PluginName            : 'Sonique Vis Example v1.0 by George Boudouris';
	                    lRequired             : VI_WAVEFORM or VI_SPECTRUM Or SONIQUEVISPROC;
	                    Initialize            : Initialize;
	                    Render                : Render;
	                    SaveSettings          : SaveSettings;
	                    OpenSettings          : OpenSettings;
                      DeInit                : DeInit;
                      Clicked               : Clicked;
                      ReceiveQueryInterface : ReceiveQueryInterface);

Var
   SaveCurrentEffect:Integer;

// DLL exports this function - it should return a pointer to a static structure as above.
// files should be renamed from .DLL to .SVP
Function QueryModule : PVisInfo; cdecl; export;

Implementation

//*====================================================================================
//	Definition of the DLL export (don't change)
//====================================================================================*/
Function QueryModule : PVisInfo;
Begin
     Result := @plugin; //  Return the Module
End;

//*====================================================================================
// Definition of the Initialize function
//
// fill in with whatever you need to initialize, this gets called before the
// plug-in starts.
//
// you can leave it empty (as is).
//====================================================================================*/

Procedure Initialize;
Begin
     // Put Here the Initialize Code
     Set8087CW($133f); { Disable all fpu exceptions }

     // Set Current Effect
     SaveCurrentEffect:=0;
End;

//*====================================================================================
//	Definition of the Render function
//
//	this is where you place your visual code,
//
//			Render should draw a whole frame into the 32 bit ABGR buffer (for blur, smoke
//			and zoom to work it should, rather than replacing the video data, add to it
//			using bitwise or, saturated add, or	alpha blend). Make sure not to exceed the
//			boundaries given. 'Pitch' specifies	the distance, in pixels, between the start
//			of each line. If you have a pointer	at the start of a line, Pointer+Pitch is
//			the start of the next line and Pointer-Pitch is the start of the previous.
//
//		this is the only function that you have to have something in.
//
//			Called for each frame. Pitch is in pixels and can be negative.
//			Render like this:
//			for (y = 0; y < height; y++)
//			{
//			   for (x = 0; x < width; x++)
//			      Video[x] = <pixel value>;
//			  Video += pitch;
//			}
//						OR
//			void PutPixel(int x, int y, unsigned long Pixel)
//			{
//			   _ASSERT( x >= 0 && x < width && y >= 0 && y < height );
//			  Video[y*pitch+x] = Pixel;
//			}
//
//		return TRUE when you've finished draw to the buffer.
//
//====================================================================================*/

Function Render0(Video:PCardinal;width,height,pitch:Integer;pVD:PVisData):Boolean ;
Var
   x, y: Integer;
   Y1Axes,Y2Axes:Integer;
   MaxY:Integer;
   Counter:Integer;
   BufferCounter:Real;
   FFTFormStep:Real;

   SaveBKColor, SaveBarColor, SaveTextColor:Cardinal;
   SaveColor:Integer;
Begin
  Try
     SaveColor     := $00FFA54A;
     SaveBKColor   := $FF shl 24 + BlueByte(SaveColor) shl 16 + GreenByte(SaveColor) shl 8 + RedByte(SaveColor);

     SaveColor     := $00FEECCD ;
     SaveBarColor  := $FF shl 24 + BlueByte(SaveColor) shl 16 + GreenByte(SaveColor) shl 8 + RedByte(SaveColor);

     SaveColor     := $00FF0000;
     SaveTextColor := $FF shl 24 + BlueByte(SaveColor) shl 16 + GreenByte(SaveColor) shl 8 + RedByte(SaveColor);

     // ************** DRAW FFT Code part START **************************
     Y1Axes:=height Shr 1-1;
     Y2Axes:=height-1;
     MaxY:=height Shr 1;
     // travel the width of the buffer
     FFTFormStep:=SONIQUE_FFTStereoBufferSize/Width; // 50=Number of Waveform Samples that the Visual Effect Needs
     BufferCounter:=0;
     For x:= 0 To Width-1 Do Begin
         // draw the left channel spectrum data, making sure that if the data is greater then half the height
	       // of the buffer, then cut it down, so no buffer overuns...  and then do the same with the right
	       // channel data. (really simple no-expense paid example)

         // Fill Screen with color
         For y:=0 to Height-1 Do PVideoBuffer(Cardinal(Video)  + Cardinal(y * pitch + x)*4)^ :=SaveBKColor; // *4 is for the cardinal size which is 32 bit = 4 bytes
         // I should be careful when accessing the pVD^.Spectrum[0,x] because
         // if  x gets higher than SONIQUE_FFTStereoBufferSize then the plugin hangs on Sonique because
         // we have a range check error
         y := Y1Axes - ((pVD^.Spectrum[0,Round(BufferCounter)]*MaxY) shr 7) ; // I can use shr 7 = Div 128 because values are not signed !!!!!!!
         if y < 0 then y := 0;
         // VideoBuffer^[y * pitch + x] := $ffcfefff;
         // Draw vert Line
         For Counter:=Y To Y1Axes Do PVideoBuffer(Cardinal(Video)  + Cardinal(Counter * pitch + x)*4)^ := SaveBarColor; // *4 is for the cardinal size which is 32 bit = 4 bytes
         y := Y2Axes - ((pVD^.Spectrum[1,Round(BufferCounter)]*MaxY) shr 7) ;  // I can use shr 7 = Div 128 because values are not signed !!!!!!!
         if (y < Y2Axes - MaxY) then y := Y2Axes - MaxY;
         // VideoBuffer^[y * pitch + x] := $ffcfefff;
         // Draw vert Line
         For Counter:=Y To Y2Axes Do PVideoBuffer(Cardinal(Video)  + Cardinal(Counter * pitch + x)*4)^ := SaveBarColor; // *4 is for the cardinal size which is 32 bit = 4 bytes

         BufferCounter:=BufferCounter+FFTFormStep;
     End;
     // draw the seperate channels line
     y:=height shr 1 ; //shr 1= Div 2;
     For x:=0 To Width-1 Do PVideoBuffer(Cardinal(Video)  + Cardinal(y * pitch + x)*4)^ :=SaveTextColor; // *4 is for the cardinal size which is 32 bit = 4 bytes

     // ************** DRAW FFT Code part END **************************
  Except

  End;
     Result:= TRUE;
End;

Function Render1(Video:PCardinal;width,height,pitch:Integer;pVD:PVisData):Boolean ;
Var
   x, y: Integer;
   Y1Axes,Y2Axes:Integer;
   MaxY:Integer;
   Counter:Integer;
   BufferCounter:Real;
   WaveFormStep:Real;

   SaveBKColor, SaveBarColor, SaveTextColor:Cardinal;
   SaveColor:Integer;
Begin
  Try
     SaveColor     := $00FFA54A;
     SaveBKColor   := $FF shl 24 + BlueByte(SaveColor) shl 16 + GreenByte(SaveColor) shl 8 + RedByte(SaveColor);

     SaveColor     := $00FEECCD ;
     SaveBarColor  := $FF shl 24 + BlueByte(SaveColor) shl 16 + GreenByte(SaveColor) shl 8 + RedByte(SaveColor);

     SaveColor     := $00FF0000;
     SaveTextColor := $FF shl 24 + BlueByte(SaveColor) shl 16 + GreenByte(SaveColor) shl 8 + RedByte(SaveColor);

     // ************** DRAW Waveform Code part START **************************
     Y1Axes:=height Shr 2-1;
     Y2Axes:=height Shr 1+height Shr 2-1;
     MaxY:=height Shr 2;

     // travel the width of the buffer
     WaveFormStep:=SONIQUE_WaveformStereoBufferSize/Width; // 50=Number of Waveform Samples that the Visual Effect Needs
     BufferCounter:=0;
     For x:= 0 To Width-1 Do Begin
         // draw the left channel spectrum data, making sure that if the data is greater then half the height
	       // of the buffer, then cut it down, so no buffer overuns...  and then do the same with the right
	       // channel data. (really simple no-expense paid example)

         // Fill Screen with color
         For y:=0 to Height-1 Do PVideoBuffer(Cardinal(Video) + Cardinal(y * pitch + x)*4)^ :=SaveBKColor; // *4 is for the cardinal size which is 32 bit = 4 bytes
         // I should be careful when accessing the pVD^.WaveForm[0,x] because
         // if  x gets higher than SONIQUE_WaveformBufferSize then the plugin hangs on Sonique because
         // we have a range check error
         // Show waveform1
         y := Y1Axes - ((pVD^.WaveForm[0,Round(BufferCounter)]*MaxY) Div 128) ; //Don't use because values are signed !!!!! shr 7 = Div 128
         if y < 0 then y := 0;
         if y > Y1Axes+MaxY then y := Y1Axes+MaxY;
         PVideoBuffer(Cardinal(Video)  + Cardinal(y * pitch + x)*4)^ := SaveBarColor; // *4 is for the cardinal size which is 32 bit = 4 bytes

         // Show waveform1
         y := Y2Axes - ((pVD^.WaveForm[1,Round(BufferCounter)]*MaxY) Div 128) ; //Don't use because values are signed !!!!! shr 7 = Div 128
         if (y < Y2Axes-MaxY) then y := Y2Axes-MaxY;
         if (y > height - 1) then y :=height-1;
         PVideoBuffer(Cardinal(Video)  + Cardinal(y * pitch + x)*4)^ := SaveBarColor; // *4 is for the cardinal size which is 32 bit = 4 bytes

         BufferCounter:=BufferCounter+WaveFormStep;
     End;
     // draw the seperate channels line
     y:=height shr 1 ; //shr 1 = Div 2
     For x:= 0 To Width-1 Do PVideoBuffer(Cardinal(Video)  + Cardinal(y * pitch + x)*4)^ :=SaveTextColor; // *4 is for the cardinal size which is 32 bit = 4 bytes

     // ************** DRAW Waveform Code part END **************************
  Except

  End;
     Result:= TRUE;
End;


Function Render(Video:PCardinal;width,height,pitch:Integer;pVD:PVisData):Boolean ;

Begin
     Case SaveCurrentEffect of
          0: Begin
                  Render0(Video,width,height,pitch,pVD);
          End;
          1: Begin
                  Render1(Video,width,height,pitch,pVD);
          End;
     End;

     Result:= TRUE;
End;

//*====================================================================================
//	Definition of the SaveSettings function
//
//		to save the settings of anything you want regarding your plugin.
//
//		use WritePrivateProfileString to save the settings.
//			ie. WritePrivateProfileString("my plugin", "brightness", "3", FileName);
//
//		return TRUE if the saving worked or FALSE if it failed.
//
//		you can leave it empty (as is).
//====================================================================================*/

Function SaveSettings(FileName:PChar):Boolean;
Begin
     Result:= TRUE;
End;

//*====================================================================================
//	Definition of the OpenSettings function
//
//		to open the settings of anything you want regarding your plugin.
//
//		use GetPrivateProfileString to save the settings.
//			ie. char BrightnessBuffer[256];
//				GetPrivateProfileString("my plugin", "brightness", "3", BrightnessBuffer, sizeof(BrightnessBuffer), FileName);
//
//		return TRUE if the opening worked or FALSE if it failed.
//
//		you can leave it empty (as is).
//====================================================================================*/

Function OpenSettings(FileName:PChar):Boolean;
Begin
     Result:= TRUE;
End;

Function DeInit:Boolean;
Begin
     // Put Here the Code
     Result:= TRUE;
End;

Function Clicked(x,y,buttons:Integer):Boolean;
Begin
     // Put Here the Code
     Inc(SaveCurrentEffect);
     If SaveCurrentEffect>1 Then SaveCurrentEffect:=0;

     Result:= TRUE;
End;

Function ReceiveQueryInterface(PluginInterface:QueryInterface): Boolean;
Begin
     // Put Here the Code
     Result:= TRUE;
End;

Begin
     // It is recommended that you disable all floating-point exceptions when
     // using OpenGL to render 3D graphics. To do this, call Set8087CW(0x133f)
     // in your main forms OnCreate event before calling any OpenGL functions.
     // Set8087CW($133f);
end.
