unit Sonique_Vis_Delphi_Header;

//====================================================================================
//
//	Sonique Visual Plugin API
//
//        (c) 1999 Sonique Team, edited by Rajesh Singh (shock@ihug.com.au)
//
//        Delphi Translation By George Boudouris 2009, (georgebou@hotmail.com)
//====================================================================================*/

// Some vis plugins can get information directly from Sonique Api
// through mapfiles using the InfoReaderInterface of the sonique
// SDK. They don't do it using this unit !!!!!!!!!!!!!!!!!!!!!!1

// Note! 
// *Video points to memory in 32bit video memory, ie, the following can be used
// for your utility and to help your understanding of the pixel format used.

// In Visual C, you should start a DLL project, under code generation set it to
// use the DLL libraries, and most likely for release you will want to turn
// optimisation for speed on. Include this .h file in your project, then create
// or add your main C/C++ file and #include "vis.h"

// To start off, fill in the VisInfo structure. Render should draw a whole frame
// into the 32 bit ABGR buffer (for blur, smoke and zoom to work it should, rather
// than replacing the video data, add to it using bitwise or, saturated add, or
// alpha blend). Make sure not to exceed the boundaries given. 'Pitch' specifies
// the distance, in pixels, between the start of each line. If you have a pointer
// at the start of a line, Pointer+Pitch is the start of the next line and
// Pointer-Pitch is the start of the previous.
// Make sure that QueryModule is defined exactly as at the end of this file,
// including the extern "C" keyword if you're using C++. It should return a
// pointer to this structure. All routines that are pointed to by the structure
// must be filled in even if they don't do anything. Render is the only function
// that is required to have a body.

interface

Type
    PVideoBuffer=PCardinal; // Cardinal is 32 bit = 4 bytes = bytes of sonique pixel color

Const
     ALPHA_MASK  =$FF000000;
     RED_MASK    =$00FF0000;
     GREEN_MASK  =$0000FF00;
     BLUE_MASK   =$000000FF;

     ALPHA_SHIFT =24;
     RED_SHIFT   =16;
     GREEN_SHIFT =8;
     BLUE_SHIFT  =0;

Function AlphaByte(x:Integer):Integer;
Function RedByte(x:Integer):Integer;
Function GreenByte(x:Integer):Integer;
Function BlueByte(x:Integer):Integer;

// Sonique Declarations
Const
     VI_WAVEFORM    =$0001; // set if you need the waveform
     VI_SPECTRUM    =$0002; // set if you need the FFT values
     SONIQUEVISPROC =$0004; // set if you want to allow Soniques user pref vis to affect your vis
                            // for example - blur, smoke and zoom

Const
     SONIQUE_WaveformStereoBufferSize=512;
     SONIQUE_FFTStereoBufferSize=256;
type
    PVisData = ^TVisData;
    TVisData=Record
       MillSec:Cardinal;		           // Sonique sets this to the time stamp of end this block of data
       Waveform: array [0..1, 0..SONIQUE_WaveformStereoBufferSize-1] of ShortInt; // Sonique sets this to the PCM data being outputted at this time
       Spectrum: array [0..1, 0..SONIQUE_FFTStereoBufferSize-1] of Byte; // Sonique sets this to a lowfidely version of the spectrum data
                                                  // being outputted at this time 0 (bass) -> 255 (treble)
    End;

// You can pass any of these strings to Query* as expression and be returned a value
// (make sure to free it later if it's a string!):
// currentsonglength, currentsongposition, scheme_visual_left, scheme_visual_right, scheme_visual_blend - int
// currentskinname, currentsongfilename, currentsongtitle, currentsongauthor, currengsongdisplaystring - str

Type
    QueryInterface = class
     private

     protected

     public
	   Function QueryInt(expression:PChar;Value:PInteger):Boolean; virtual;
	   Function QueryString(expression:PChar):PChar; virtual;
           Procedure FreeString(SaveString:PChar); virtual;
     end;

type
    PVisInfo=^TVisInfo;
    TVisInfo=Record
	Version               : Cardinal;          // 1 = supports Clicked(x,y,buttons)
	PluginName            : PChar;             // Set to the name of the plugin
	lRequired             : LongInt;	   // Which vis data this plugin requires (set to a combination of
                                                   // the VI_WAVEFORM, VI_SPECTRUM and SONIQEUVISPROC flags)
        Initialize            : procedure; cdecl;  // Called some time before your plugin is asked to render for the first time
        Render                : Function (Video:PCardinal;width,height,pitch:Integer;pVD:PVisData) : Boolean ; cdecl;
                                                   // Called for each frame. Pitch is in pixels and can be negative.
						   // Render like this:
						   // for (y = 0; y < height; y++)
						   // {
						   //    for (x = 0; x < width; x++)
						   //       Video[x] = <pixel value>;
						   //	  Video += pitch;
						   // }
						   //				OR
						   // void PutPixel(int x, int y, unsigned long Pixel)
						   // {
						   //    _ASSERT( x >= 0 && x < width && y >= 0 && y < height );
						   //	  Video[y*pitch+x] = Pixel;
						   // }
	SaveSettings          : Function(FileName:PChar): Boolean; cdecl;
						   // Use WritePrivateProfileString to save settings when this is called
						   // Example:
						   // WritePrivateProfileString("my plugin", "brightness", "3", FileName);
	OpenSettings          : Function(FileName:PChar): Boolean; cdecl;
						   // Use GetPrivateProfileString similarly:
						   // char BrightnessBuffer[256];
						   // GetPrivateProfileString("my plugin", "brightness", "3", BrightnessBuffer, sizeof(BrightnessBuffer), FileName);
        Deinit                : Function: Boolean; cdecl;
	Clicked               : Function(x,y,buttons:Integer) : Boolean; cdecl;
        ReceiveQueryInterface : Function(PluginInterface:QueryInterface): Boolean; cdecl;
End;

implementation

Function AlphaByte(x:Integer):Integer;
Begin
     Result:=x Shr ALPHA_SHIFT;
End;

Function RedByte(x:Integer):Integer;
Begin
     Result:=(x And RED_MASK) Shr RED_SHIFT;
End;

Function GreenByte(x:Integer):Integer;
Begin
      Result:=(x And GREEN_MASK) Shr GREEN_SHIFT;
End;

Function BlueByte(x:Integer):Integer;
Begin
     Result:=(x And BLUE_MASK) Shr BLUE_SHIFT;
End;

Function QueryInterface.QueryInt(expression:PChar;Value:PInteger):Boolean;
Begin

End;

Function QueryInterface.QueryString(expression:PChar):PChar; 
Begin

End;

Procedure QueryInterface.FreeString(SaveString:PChar);
Begin

End;

end.
